"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.JWTGuard = exports.JWTToken = void 0;
const Base_1 = require("@adonisjs/auth/build/src/Guards/Base");
const luxon_1 = require("luxon");
const helpers_1 = require("@poppinss/utils/build/helpers");
const crypto_1 = require("crypto");
const ProviderToken_1 = require("@adonisjs/auth/build/src/Tokens/ProviderToken");
const sign_1 = require("jose/jwt/sign");
const verify_1 = require("jose/jwt/verify");
const standalone_1 = require("@adonisjs/auth/build/standalone");
const JwtAuthenticationException_1 = __importDefault(require("../Exceptions/JwtAuthenticationException"));
/**
 * JWT token represents a persisted token generated for a given user.
 *
 * Calling `token.toJSON()` will give you an object, that you can send back
 * as response to share the token with the client.
 */
class JWTToken {
    constructor(name, // Name associated with the token
    accessToken, // The raw token value. Only available for the first time
    refreshToken, // The raw refresh token value. Only available for the first time
    user) {
        this.name = name;
        this.accessToken = accessToken;
        this.refreshToken = refreshToken;
        this.user = user;
        /**
         * The type of the token. Always set to bearer
         */
        this.type = 'bearer';
    }
    /**
     * Shareable version of the token
     */
    toJSON() {
        return {
            type: this.type,
            token: this.accessToken,
            refreshToken: this.refreshToken,
            ...(this.expiresAt ? { expires_at: this.expiresAt.toISO() || undefined } : {}),
            ...(this.expiresIn ? { expires_in: this.expiresIn } : {}),
        };
    }
}
exports.JWTToken = JWTToken;
/**
 * Exposes the API to generate and authenticate HTTP request using jwt tokens
 */
class JWTGuard extends Base_1.BaseGuard {
    /**
     * constructor of class.
     */
    constructor(_name, config, emitter, provider, ctx, tokenProvider) {
        super('jwt', config, provider);
        this.config = config;
        this.emitter = emitter;
        this.ctx = ctx;
        this.tokenProvider = tokenProvider;
        /**
         * Token type for the persistance store
         */
        this.tokenType = this.config.tokenProvider.type || 'jwt_token';
    }
    /**
     * Verify user credentials and perform login
     */
    async attempt(uid, password, options) {
        const user = await this.verifyCredentials(uid, password);
        return this.login(user, options);
    }
    /**
     * Same as [[authenticate]] but returns a boolean over raising exceptions
     */
    async check() {
        try {
            await this.authenticate();
        }
        catch (error) {
            /**
             * Throw error when it is not an instance of the authentication
             */
            if (!(error instanceof standalone_1.AuthenticationException)) {
                throw error;
            }
            this.ctx.logger.trace(error, 'Authentication failure');
        }
        return this.isAuthenticated;
    }
    /**
     * Authenticates the current HTTP request by checking for the bearer token
     */
    async authenticate() {
        /**
         * Return early when authentication has already attempted for
         * the current request
         */
        if (this.authenticationAttempted) {
            return this.user;
        }
        this.authenticationAttempted = true;
        /**
         * Ensure the "Authorization" header value exists
         */
        const token = this.getBearerToken();
        const tokenValue = this.parsePublicToken(token);
        /**
         * Query token and user
         */
        const providerToken = await this.getProviderToken(tokenValue);
        const providerUser = await this.getUserById(providerToken.userId);
        /**
         * Marking user as logged in
         */
        this.markUserAsLoggedIn(providerUser.user, true);
        this.token = providerToken;
        /**
         * Emit authenticate event. It can be used to track user logins.
         */
        this.emitter.emit('adonis:api:authenticate', this.getAuthenticateEventData(providerUser.user, this.token));
        return providerUser.user;
    }
    /**
     * Generate token for a user. It is merely an alias for `login`
     */
    async generate(user, options) {
        return this.login(user, options);
    }
    /**
     * Login user using their id
     */
    async loginViaId(id, options) {
        const providerUser = await this.findById(id);
        return this.login(providerUser.user, options);
    }
    /**
     * Login a user
     */
    async login(user, options) {
        /**
         * Normalize options with defaults
         */
        let { expiresIn, name, payload, ...meta } = Object.assign({ name: 'JWT Access Token' }, options);
        /**
         * Since the login method is not exposed to the end user, we cannot expect
         * them to instantiate and pass an instance of provider user, so we
         * create one manually.
         */
        const providerUser = await this.getUserForLogin(user, this.config.provider.identifierKey);
        /**
         * "getUserForLogin" raises exception when id is missing, so we can
         * safely assume it is defined
         */
        const userId = providerUser.getId();
        if (payload) {
            payload.userId = userId;
        }
        else {
            payload = {
                userId: userId,
                user: {
                    name: user.name,
                    email: user.email,
                },
            };
        }
        const token = await this.generateTokenForPersistance(expiresIn, payload);
        /**
         * Persist token to the database. Make sure that we are always
         * passing the hash to the storage driver
         */
        const providerToken = new ProviderToken_1.ProviderToken(name, token.accessTokenHash, userId, this.tokenType);
        providerToken.expiresAt = token.expiresAt;
        meta[this.config.tokenProvider.refreshTokenKey] = token.refreshTokenHash;
        providerToken.meta = meta;
        await this.tokenProvider.write(providerToken);
        /**
         * Construct a new API Token instance
         */
        const apiToken = new JWTToken(name, token.accessToken, token.refreshToken, providerUser.user);
        apiToken.tokenHash = token.accessTokenHash;
        apiToken.expiresAt = token.expiresAt;
        apiToken.meta = meta;
        /**
         * Marking user as logged in
         */
        this.markUserAsLoggedIn(providerUser.user);
        this.token = providerToken;
        /**
         * Emit login event. It can be used to track user logins.
         */
        this.emitter.emit('adonis:api:login', this.getLoginEventData(providerUser.user, apiToken));
        return apiToken;
    }
    /**
     * Logout by removing the token from the storage
     */
    async logout(_options) {
        if (!this.authenticationAttempted) {
            await this.check();
        }
        /**
         * Clean up token from storage
         */
        if (this.parsedToken) {
            await this.tokenProvider.destroyWithHash(this.parsedToken, this.tokenType);
        }
        this.markUserAsLoggedOut();
    }
    /**
     * Alias for the logout method
     */
    revoke() {
        return this.logout();
    }
    /**
     * Serialize toJSON for JSON.stringify
     */
    toJSON() {
        return {
            isLoggedIn: this.isLoggedIn,
            isGuest: this.isGuest,
            authenticationAttempted: this.authenticationAttempted,
            isAuthenticated: this.isAuthenticated,
            user: this.user,
        };
    }
    /**
     * Generates a new access token + refresh token + hash's for the persistance.
     */
    async generateTokenForPersistance(expiresIn, payload = {}) {
        let builder = new sign_1.SignJWT({ data: payload })
            .setProtectedHeader({ alg: 'RS256' })
            .setIssuedAt();
        if (this.config.issuer) {
            builder = builder.setIssuer(this.config.issuer);
        }
        if (this.config.audience) {
            builder = builder.setAudience(this.config.audience);
        }
        if (expiresIn) {
            builder = builder.setExpirationTime(expiresIn);
        }
        const accessToken = await builder.sign(this.generateKey(this.config.privateKey));
        const accessTokenHash = this.generateHash(accessToken);
        let refreshTokenBuilder = new sign_1.SignJWT({ data: accessTokenHash })
            .setProtectedHeader({ alg: 'RS256' })
            .setIssuedAt();
        if (this.config.issuer) {
            refreshTokenBuilder = refreshTokenBuilder.setIssuer(this.config.issuer);
        }
        if (this.config.audience) {
            refreshTokenBuilder = refreshTokenBuilder.setAudience(this.config.audience);
        }
        const refreshToken = await refreshTokenBuilder.sign(this.generateKey(this.config.privateKey));
        return {
            accessToken,
            accessTokenHash,
            refreshToken,
            refreshTokenHash: this.generateHash(refreshToken),
            expiresAt: this.getExpiresAtDate(expiresIn),
        };
    }
    /**
     * Converts key string to Buffer
     */
    generateKey(hash) {
        return (0, crypto_1.createPrivateKey)(Buffer.from(hash));
    }
    /**
     * Converts value to a sha256 hash
     */
    generateHash(token) {
        return (0, crypto_1.createHash)('sha256').update(token).digest('hex');
    }
    /**
     * Converts expiry duration to an absolute date/time value
     */
    getExpiresAtDate(expiresIn) {
        if (!expiresIn) {
            return;
        }
        const milliseconds = typeof expiresIn === 'string' ? helpers_1.string.toMs(expiresIn) : expiresIn;
        return luxon_1.DateTime.local().plus({ milliseconds });
    }
    /**
     * Returns the bearer token
     */
    getBearerToken() {
        /**
         * Ensure the "Authorization" header value exists
         */
        const token = this.ctx.request.header('Authorization');
        if (!token) {
            throw new JwtAuthenticationException_1.default("No Authorization header passed");
        }
        /**
         * Ensure that token has minimum of two parts and the first
         * part is a constant string named `bearer`
         */
        const [type, value] = token.split(' ');
        if (!type || type.toLowerCase() !== 'bearer' || !value) {
            throw new JwtAuthenticationException_1.default("Invalid Authorization header value: " + token);
        }
        return value;
    }
    /**
     * Parses the token received in the request. The method also performs
     * some initial level of sanity checks.
     */
    parsePublicToken(token) {
        const parts = token.split('.');
        /**
         * Ensure the token has at least three parts
         */
        if (parts.length < 3) {
            throw new JwtAuthenticationException_1.default("Invalid JWT format");
        }
        /**
         * Ensure 2nd part of the token has the expected length
         */
        const value = parts.join('.');
        if (value.length < 30) {
            throw new JwtAuthenticationException_1.default("Invalid JWT format: token too short");
        }
        /**
         * Set parsed token
         */
        this.parsedToken = token;
        return token;
    }
    /**
     * Returns the token by reading it from the token provider
     */
    async getProviderToken(value) {
        const providerToken = await this.tokenProvider.read("", this.generateHash(value), this.tokenType);
        if (!providerToken) {
            throw new JwtAuthenticationException_1.default("Invalid JWT token");
        }
        return providerToken;
    }
    /**
     * Returns user from the user session id
     */
    async getUserById(id) {
        const token = this.parsedToken || '';
        const secret = this.generateKey(this.config.privateKey);
        const { payload } = await (0, verify_1.jwtVerify)(token, secret, {
            issuer: this.config.issuer,
            audience: this.config.audience,
        });
        const { data, exp } = payload;
        if (exp && exp < Math.floor(luxon_1.DateTime.now().toSeconds())) {
            throw new JwtAuthenticationException_1.default("Expired JWT token");
        }
        if (!data) {
            throw new JwtAuthenticationException_1.default("Invalid JWT payload");
        }
        if (data.userId !== id) {
            throw new JwtAuthenticationException_1.default("Invalid user in payload");
        }
        const authenticatable = await this.provider.findById(data.userId);
        if (!authenticatable.user) {
            throw new JwtAuthenticationException_1.default("No user found from paypload");
        }
        return authenticatable;
    }
    /**
     * Returns data packet for the login event. Arguments are
     *
     * - The mapping identifier
     * - Logged in user
     * - HTTP context
     * - API token
     */
    getLoginEventData(user, token) {
        return {
            name: this.name,
            ctx: this.ctx,
            user,
            token,
        };
    }
    /**
     * Returns data packet for the authenticate event. Arguments are
     *
     * - The mapping identifier
     * - Logged in user
     * - HTTP context
     * - A boolean to tell if logged in viaRemember or not
     */
    getAuthenticateEventData(user, token) {
        return {
            name: this.name,
            ctx: this.ctx,
            user,
            token,
        };
    }
}
exports.JWTGuard = JWTGuard;
